unit ctlQRCrossTab;

interface

uses
  Windows, Messages, SysUtils, Classes, Graphics, Controls, Forms, Dialogs,
  quickrpt, Qrctrls, DB;

type
  TDataCell = class
    Title : string;
    QRLabel : TQRLabel;
  end;
  TYTotalCell = class
    Total : double;
    QRLabel : TQRLabel;
  end;

  TQRCrossTab = class(TComponent)
  private
    XWidth : integer;
    XLabels : TStringList;
    YLabels : TStringList;
    LineNo : integer;
    YLabel : TQRLabel;
    TitleLabel : TQRLabel;
    CellLabel : TQRLabel;
    XTotalLabel : TQRLabel;
    XTotalCellLabel : TQRLabel;
    YTotalCellLabel : TQRLabel;
    GrandTotalLabel : TQRLabel;
    DataLabels : TList;
    YTotals : TList;
    ADataSet : TDataSet;
    XField : string;
    YField : string;
    SumField : string;
    QReport : TQuickRep;
    CHBand : TQRBand;
    DTBand : TQRBand;
    SMBand : TQRBand;
    FormatString : string;
    PrintBlank : boolean;

    procedure NeedData(Sender : TObject; var MoreData : Boolean);
    procedure AfterPreview(Sender: TObject);
    procedure BeforePrint(Sender: TQRCustomBand;  var PrintBand: Boolean);

  protected
    { Protected declarations }
  public
    procedure Preview;

  published
    property CellWidth : integer read XWidth write XWidth;
    property XFieldName : string read XField write XField;
    property YFieldName : string read YField write YField;
    property SummaryFieldName : string read SumField write SumField;
    property QuickReport : TQuickRep read QReport write QReport;
    property YTitle : TQRLabel read YLabel write YLabel;
    property SampleTitle : TQRLabel read TitleLabel write TitleLabel;
    property SampleCell : TQRLabel read CellLabel write CellLabel;
    property XTotalTitle : TQRLabel read XTotalLabel write XTotalLabel;
    property SampleXTotalCell : TQRLabel read XTotalCellLabel write XTotalCellLabel;
    property SampleYTotalCell : TQRLabel read YTotalCellLabel write YTotalCellLabel;
    property GandTotalCell : TQRLabel read GrandTotalLabel write GrandTotalLabel;
    property ColumnHeaderBand : TQRBand read CHBand write CHBand;
    property DetailBand : TQRBand read DTBand write DTBand;
    property SummaryBand : TQRBand read SMBand write SMBand;
    property DataSet : TDataSet read ADataSet write ADataSet;
    property CellFormat : string read FormatString write FormatString;
    property ZeroAsBlank : boolean read PrintBlank write PrintBlank;
  end;

procedure Register;

implementation

procedure Register;
begin
  RegisterComponents('QReport', [TQRCrossTab]);
end;

procedure TQRCrossTab.Preview;
var
  n, x : integer;
  NewDataCell : TDataCell;
  NewDataLabel : TQRLabel;
  NewYTotalCell : TYTotalCell;

begin
  TitleLabel.Enabled := False;
  CellLabel.Enabled := False;
  YTotalCellLabel.Enabled := False;
  
  XLabels := TStringList.Create;
  YLabels := TStringList.Create;
  DataLabels := TList.Create;
  YTotals := TList.Create;

  XLabels.Sorted := True;
  YLabels.Sorted := True;

  ADataSet.First;
  while not ADataSet.eof do begin
    if not XLabels.Find(ADataSet.FieldByName(XField).AsString,n) then
      XLabels.Add(ADataSet.FieldByName(XField).AsString);
    if not YLabels.Find(ADataSet.FieldByName(YField).AsString,n) then
      YLabels.Add(ADataSet.FieldByName(YField).AsString);

    ADataSet.Next;
  end;

  x := XWidth;
  for n := 0 to XLabels.Count-1 do begin
    with TQRLabel.Create(QReport) do begin
      Parent := CHBand;
      Left := x;
      Alignment := TitleLabel.Alignment;
      AutoSize := TitleLabel.AutoSize;
      AutoStretch := TitleLabel.AutoStretch;
      Color := TitleLabel.Color;
      Font.Color := TitleLabel.Font.Color;
      Font.Height := TitleLabel.Font.Height;
      Font.Name := TitleLabel.Font.Name;
      Font.Pitch := TitleLabel.Font.Pitch;
      Font.Size := TitleLabel.Font.Size;
      Font.Style := TitleLabel.Font.Style;
      Frame.Color := TitleLabel.Frame.Color;
      Frame.DrawBottom := TitleLabel.Frame.DrawBottom;
      Frame.DrawLeft := TitleLabel.Frame.DrawLeft;
      Frame.DrawRight := TitleLabel.Frame.DrawRight;
      Frame.DrawTop := TitleLabel.Frame.DrawTop;
      Frame.Style := TitleLabel.Frame.Style;
      Frame.Width := TitleLabel.Frame.Width;
      Height := TitleLabel.Height;
      ParentFont := TitleLabel.ParentFont;
      Top := TitleLabel.Top;
      Transparent := TitleLabel.Transparent;
      Width := TitleLabel.Width;
      WordWrap := TitleLabel.WordWrap;
      Caption := XLabels[n];
    end;

    NewDataCell := TDataCell.Create;
    NewDataLabel := TQRLabel.Create(QReport);
    with NewDataLabel do begin
      Parent := DTBand;
      Left := X;

      Alignment := CellLabel.Alignment;
      AutoSize := CellLabel.AutoSize;
      AutoStretch := CellLabel.AutoStretch;
      Color := CellLabel.Color;
      Font.Color := CellLabel.Font.Color;
      Font.Height := CellLabel.Font.Height;
      Font.Name := CellLabel.Font.Name;
      Font.Pitch := CellLabel.Font.Pitch;
      Font.Size := CellLabel.Font.Size;
      Font.Style := CellLabel.Font.Style;
      Frame.Color := CellLabel.Frame.Color;
      Frame.DrawBottom := CellLabel.Frame.DrawBottom;
      Frame.DrawLeft := CellLabel.Frame.DrawLeft;
      Frame.DrawRight := CellLabel.Frame.DrawRight;
      Frame.DrawTop := CellLabel.Frame.DrawTop;
      Frame.Style := CellLabel.Frame.Style;
      Frame.Width := CellLabel.Frame.Width;
      Height := CellLabel.Height;
      ParentFont := CellLabel.ParentFont;
      Top := CellLabel.Top;
      Transparent := CellLabel.Transparent;
      Width := CellLabel.Width;
      WordWrap := CellLabel.WordWrap;
      Caption := '';
    end;
    NewDataCell.Title := XLabels[n];
    NewDataCell.QRLabel := NewDataLabel;
    DataLabels.Add(NewDataCell);

    NewYTotalCell := TYTotalCell.Create;
    NewDataLabel := TQRLabel.Create(QReport);
    with NewDataLabel do begin
      Parent := SMBand;
      Left := X;

      Alignment := YTotalCellLabel.Alignment;
      AutoSize := YTotalCellLabel.AutoSize;
      AutoStretch := YTotalCellLabel.AutoStretch;
      Color := YTotalCellLabel.Color;
      Font.Color := YTotalCellLabel.Font.Color;
      Font.Height := YTotalCellLabel.Font.Height;
      Font.Name := YTotalCellLabel.Font.Name;
      Font.Pitch := YTotalCellLabel.Font.Pitch;
      Font.Size := YTotalCellLabel.Font.Size;
      Font.Style := YTotalCellLabel.Font.Style;
      Frame.Color := YTotalCellLabel.Frame.Color;
      Frame.DrawBottom := YTotalCellLabel.Frame.DrawBottom;
      Frame.DrawLeft := YTotalCellLabel.Frame.DrawLeft;
      Frame.DrawRight := YTotalCellLabel.Frame.DrawRight;
      Frame.DrawTop := YTotalCellLabel.Frame.DrawTop;
      Frame.Style := YTotalCellLabel.Frame.Style;
      Frame.Width := YTotalCellLabel.Frame.Width;
      Height := YTotalCellLabel.Height;
      ParentFont := YTotalCellLabel.ParentFont;
      Top := YTotalCellLabel.Top;
      Transparent := YTotalCellLabel.Transparent;
      Width := YTotalCellLabel.Width;
      WordWrap := YTotalCellLabel.WordWrap;
      Caption := '';
    end;
    NewYTotalCell.Total := 0.0;
    NewYTotalCell.QRLabel := NewDataLabel;
    YTotals.Add(NewYTotalCell);

    x := x + XWidth;
  end;
  XTotalLabel.Left := x;
  XTotalCellLabel.Left := x;
  GrandTotalLabel.Left := x;
  LineNo := 0;
  SMBand.BeforePrint := BeforePrint;
  QReport.OnNeedData := NeedData;
  QReport.AfterPreview := AfterPreview;
  QReport.Preview;

end;

procedure TQRCrossTab.NeedData(Sender : TObject; var MoreData : Boolean);
var
  n : integer;
  XTotal : double;

begin
  if LineNo < YLabels.Count then begin
    for n := 0 to DataLabels.Count -1 do begin
      if PrintBlank then
        TDataCell(DataLabels[n]).QRLabel.Caption := ''
      else
        TDataCell(DataLabels[n]).QRLabel.Caption := Format(FormatString,[0.0]);
    end;

    YLabel.Caption := YLabels[LineNo];
    DataSet.Filtered := False;
    DataSet.Filter := YField + ' = ''' + YLabels[LineNo] + '''';
    DataSet.Filtered := True;
    DataSet.First;
    XTotal := 0.0;
    while not DataSet.eof do begin
      for n := 0 to DataLabels.Count -1 do begin
        if TDataCell(DataLabels[n]).Title = DataSet.FieldByName(XField).AsString then begin
          TQRLabel(TDataCell(DataLabels[n]).QRLabel).Caption := Format(FormatString,[DataSet.FieldByName(SumField).AsFloat]);
          XTotal := XTotal + DataSet.FieldByName(SumField).AsFloat;
          TYTotalCell(YTotals[n]).Total := TYTotalCell(YTotals[n]).Total + DataSet.FieldByName(SumField).AsFloat;
          Break;
        end;
      end;
      DataSet.Next;
    end;
    XTotalCellLabel.Caption := Format(FormatString,[XTotal]);
    inc(LineNo);
    MoreData := True;
  end
  else begin
    LineNo := 0;
    MoreData := False;
  end;
end;

procedure TQRCrossTab.AfterPreview(Sender: TObject);
begin
  XLabels.Free;
  YLabels.Free;
  DataLabels.Free;
  YTotals.Free;
end;

procedure TQRCrossTab.BeforePrint(Sender: TQRCustomBand;  var PrintBand: Boolean);
var
  n : integer;
  XTotal : double;

begin
  XTotal := 0.0;
  for n := 0 to YTotals.Count -1 do begin
    TYTotalCell(YTotals[n]).QRLabel.Caption := Format(FormatString,[TYTotalCell(YTotals[n]).Total]);
    XTotal := XTotal + TYTotalCell(YTotals[n]).Total;
    TYTotalCell(YTotals[n]).Total := 0.0;
  end;
  GrandTotalLabel.Caption := Format(FormatString,[XTotal]);
end;

end.
